pub struct ShardingSpec {
pub spec_id: u32,
pub fields: Vec<ShardingField>,
}Expand description
Sharding spec definition.
Fields
spec_id: u32fields: Vec<ShardingField>Trait Implementations
Sourceimpl Clone for ShardingSpec
impl Clone for ShardingSpec
Sourcefn clone(&self) -> ShardingSpec
fn clone(&self) -> ShardingSpec
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for ShardingSpec
impl Debug for ShardingSpec
Sourceimpl DeepSizeOf for ShardingSpec
impl DeepSizeOf for ShardingSpec
Sourcefn deep_size_of_children(&self, context: &mut Context) -> usize
fn deep_size_of_children(&self, context: &mut Context) -> usize
Returns an estimation of the heap-managed storage of this object.
This does not include the size of the object itself. Read more
Sourcefn deep_size_of(&self) -> usize
fn deep_size_of(&self) -> usize
Returns an estimation of a total size of memory owned by the
object, including heap-managed storage. Read more
Sourceimpl<'de> Deserialize<'de> for ShardingSpec
impl<'de> Deserialize<'de> for ShardingSpec
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl From<&ShardingSpec> for ShardingSpec
impl From<&ShardingSpec> for ShardingSpec
Sourcefn from(rs: &ShardingSpec) -> Self
fn from(rs: &ShardingSpec) -> Self
Converts to this type from the input type.
Sourceimpl From<ShardingSpec> for ShardingSpec
impl From<ShardingSpec> for ShardingSpec
Sourcefn from(rs: ShardingSpec) -> Self
fn from(rs: ShardingSpec) -> Self
Converts to this type from the input type.
Sourceimpl PartialEq for ShardingSpec
impl PartialEq for ShardingSpec
Sourcefn eq(&self, other: &ShardingSpec) -> bool
fn eq(&self, other: &ShardingSpec) -> bool
Tests for
self and other values to be equal, and is used by ==.Sourceimpl Serialize for ShardingSpec
impl Serialize for ShardingSpec
impl Eq for ShardingSpec
impl StructuralPartialEq for ShardingSpec