InvertedIndex

Struct InvertedIndex 

pub struct InvertedIndex { /* private fields */ }

Implementations

Source

impl InvertedIndex

pub fn tokenizer(&self) -> Box<dyn LanceTokenizer>

pub fn params(&self) -> &InvertedIndexParams

pub fn partition_count(&self) -> usize

Returns the number of partitions in this inverted index.

pub fn deleted_fragments(&self) -> &RoaringBitmap

Returns the set of fragments which are contained in the index, but no longer in the dataset.

Most other indices remove data from deleted fragments when the index updates (copy-on-write). However, this would require an expensive copy of the FTS index. Instead, we track the deleted fragments and prune them at search time (merge-on-read).

pub async fn merge_segments( segments: &[Arc<Self>], new_data: SendableRecordBatchStream, dest_store: &dyn IndexStore, old_data_filter: Option<OldIndexDataFilter>, progress: Arc<dyn IndexBuildProgress>, ) -> Result<CreatedIndex>

pub fn bm25_base_scorer(&self, query_tokens: &Tokens) -> MemBM25Scorer

pub fn bm25_stats_for_terms(&self, terms: &[String]) -> (u64, usize, Vec<usize>)

pub fn expand_fuzzy_tokens( &self, tokens: &Tokens, params: &FtsSearchParams, ) -> Result<Tokens>

Expand fuzzy query tokens against all partitions in this segment.

Search documents that match the query and return row ids sorted by BM25 score.

When base_scorer is provided, search uses those corpus-level BM25 statistics instead of deriving them from this segment alone.

pub fn is_legacy(&self) -> bool

pub async fn load( store: Arc<dyn IndexStore>, frag_reuse_index: Option<Arc<FragReuseIndex>>, index_cache: &LanceCache, ) -> Result<Arc<Self>>
where Self: Sized,

Source

impl InvertedIndex

pub async fn prewarm_with_options( &self, options: &FtsPrewarmOptions, ) -> Result<()>