InvertedIndexParams

Struct InvertedIndexParams 

pub struct InvertedIndexParams { /* private fields */ }
Tokenizer configs

Implementations

Source

impl InvertedIndexParams

pub fn new(base_tokenizer: String, language: Language) -> Self

Create a new InvertedIndexParams with the given base tokenizer and language.

The base_tokenizer can be one of the following:

  • simple: splits tokens on whitespace and punctuation, default
  • whitespace: splits tokens on whitespace
  • raw: no tokenization
  • ngram: N-Gram tokenizer
  • lindera/*: Lindera tokenizer
  • jieba/*: Jieba tokenizer

The language is used for stemming and removing stop words, this is not used for lindera/* and jieba/* tokenizers. Default to English.

pub fn lance_tokenizer(self, lance_tokenizer: String) -> Self

pub fn base_tokenizer(self, base_tokenizer: String) -> Self

pub fn language(self, language: &str) -> Result<Self>

pub fn with_position(self, with_position: bool) -> Self

Set whether to store the position of the term in the document. This can significantly increase the size of the index. If false, only store the frequency of the term in the document. This doesn’t work with ngram tokenizer. Default to false.

pub fn has_positions(&self) -> bool

Get whether positions are stored in this index.

pub fn max_token_length(self, max_token_length: Option<usize>) -> Self

pub fn lower_case(self, lower_case: bool) -> Self

pub fn stem(self, stem: bool) -> Self

pub fn remove_stop_words(self, remove_stop_words: bool) -> Self

pub fn custom_stop_words(self, custom_stop_words: Option<Vec<String>>) -> Self

pub fn ascii_folding(self, ascii_folding: bool) -> Self

pub fn ngram_min_length(self, min_length: u32) -> Self

Set the minimum N-Gram length, only works when base_tokenizer is ngram. Must be greater than 0 and not greater than max_ngram_length. Default to 3.

pub fn ngram_max_length(self, max_length: u32) -> Self

Set the maximum N-Gram length, only works when base_tokenizer is ngram. Must be greater than 0 and not less than min_ngram_length. Default to 3.

pub fn ngram_prefix_only(self, prefix_only: bool) -> Self

Set whether only prefix N-Gram is generated, only works when base_tokenizer is ngram. Default to false.

pub fn skip_merge(self, skip_merge: bool) -> Self

Skip merging partitions after indexing.

pub fn build(&self) -> Result<Box<dyn LanceTokenizer>>