pub struct TwoFileShuffler { /* private fields */ }Expand description
A shuffler that writes all data to just two files (data + offsets) instead of one file per partition. This avoids hitting OS file descriptor limits when there are many partitions.
First we accumulate data in memory until we reach the batch size limit. Then we sort the data by partition ID and compute an offset per partition. Then we write the data to a data file and the offsets to an offsets file.
To read the data back, we read every Nth value from the offsets file to get the start and end of each partition.
Then we read those ranges from the data file.
Implementations
Sourceimpl TwoFileShuffler
impl TwoFileShuffler
pub fn new(output_dir: Path, num_partitions: usize) -> Self
pub fn with_progress(self, progress: Arc<dyn IndexBuildProgress>) -> Self